/*
 * Copyright (c) 2016-2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include "sa_common.h"
#include "sa_types.h"
#include "ipc.h"
#include "ipc_config.h"
#include "ipc_server.h"
#include "json_util.h"
#include "setup_system.h"

//////////////////////////////////////////////////////////////////
// callee function body
//////////////////////////////////////////////////////////////////

char *__device_reboot(char *data)
{
	_D("[SERVER] __device_reboot");

	int ret = 0;
	char *buf = NULL;
	char *ret_buf = NULL;
	int buf_len = 0;

	ret = sa_device_reboot();
	_D("[SERVER] sa_create_device_reboot_json_string()");
	buf = sa_create_device_reboot_json_string(ret);
	if (buf != NULL) {
		buf_len = strlen(buf) + 1;
		ret_buf = (char *)malloc(buf_len);
		if (ret_buf != NULL)
			snprintf(ret_buf, buf_len, "%s", buf);
		free(buf);
	}
	//_D("ret_buf : %s", ret_buf);
	return ret_buf;
}

char *__get_os_info(void)
{
	_D("[SERVER] __get_os_info");

	os_info_s *os_info_h = NULL;
	char *ret_buf = NULL;
	int ret = 0;
	char *buf = NULL;
	int buf_len = 0;

	os_info_h = (os_info_s *) malloc(sizeof(os_info_s));
	if (os_info_h != NULL) {
		memset(os_info_h, 0x0, sizeof(os_info_s));
		ret = sa_get_os_info(os_info_h);
		if (ret != -1) {
			// make json string
			buf = sa_create_os_info_json_string(os_info_h);
			//_D("json_str : %s", buf);
			if (buf != NULL) {
				buf_len = strlen(buf) + 1;
				ret_buf = (char *)malloc(buf_len);
				if (ret_buf != NULL)
					snprintf(ret_buf, buf_len, "%s", buf);
				free(buf);
			}
		}

		if (os_info_h->platformVer)
			free(os_info_h->platformVer);
		if (os_info_h->baseOSVer)
			free(os_info_h->baseOSVer);
		if (os_info_h->dockerVer)
			free(os_info_h->dockerVer);
		free(os_info_h);
	}
	//_D("ret_buf : %s", ret_buf);
	return ret_buf;
}

char *__get_disk_info(void)
{
	_D("[SERVER] __get_disk_info");
	char *ret_buf = NULL;
	int ret = 0;
	disk_info_s *disk_info_h = NULL;
	int index;
	char *buf = NULL;
	int buf_len = 0;

	disk_info_h = (disk_info_s *) malloc(sizeof(disk_info_s));
	if (disk_info_h != NULL) {
		memset(disk_info_h, 0x0, sizeof(disk_info_s));
		ret = sa_get_disk_info(disk_info_h);
		if (ret != -1) {
			// make json string
			buf = sa_create_disk_info_json_string(disk_info_h);
			//_D("json_str : %s", buf);
			if (buf != NULL) {
				buf_len = strlen(buf) + 1;
				ret_buf = (char *)malloc(buf_len);
				if (ret_buf != NULL)
					snprintf(ret_buf, buf_len, "%s", buf);
				free(buf);
			}
		}

		for (index = 0; index < disk_info_h->count; index++) {
			if (disk_info_h->disk[index].path)
				free(disk_info_h->disk[index].path);
		}
		free(disk_info_h);
	}
	//_D("ret_buf : %s", ret_buf);
	return ret_buf;
}

char *__factory_restore(char *data)
{
	_D("[SERVER] __factory_restore");

	int ret = 0;
	char *buf = NULL;
	char *ret_buf = NULL;
	int buf_len = 0;
	int fdevice_restart = 1;

	ret = sa_factory_restore(fdevice_restart);
	_D("[SERVER] sa_create_factory_restore_json_string()");
	buf = sa_create_factory_restore_json_string(ret);
	if (buf != NULL) {
		buf_len = strlen(buf) + 1;
		ret_buf = (char *)malloc(buf_len);
		if (ret_buf != NULL)
			snprintf(ret_buf, buf_len, "%s", buf);
		free(buf);
	}
	//_D("ret_buf : %s", ret_buf);
	return ret_buf;
}

//////////////////////////////////////////////////////////////////
// callee function list structure & register
//////////////////////////////////////////////////////////////////

typedef void *(*cmdFunctionPtr) (void *, ...);

struct _cmd_table_s {
	char cmd[50];
	cmdFunctionPtr func;
};

typedef struct _cmd_table_s cmd_mapping_tables;

cmd_mapping_tables cmd_mapping_table[] = {
	{"device_reboot", (cmdFunctionPtr) __device_reboot},
	{"get_os_info", (cmdFunctionPtr) __get_os_info},
	{"get_disk_info", (cmdFunctionPtr) __get_disk_info},
	{"factory_restore", (cmdFunctionPtr) __factory_restore},
	{"end", NULL}
};

char *__interface_parser(char *data, int size)
{
	char *func = NULL;
	char *param = NULL;
	char *ret_buf = NULL;
	int i = 0;

	_D("interface_parser");

	func = json_getString(data, "func");
	param = json_getObject(data, "param");

	_D("func:%s", func);
	_D("param:%s", param);

	while (1) {
		if (!strncmp(cmd_mapping_table[i].cmd, "end", strlen("end")))
			break;

		if (func != NULL && !strncmp(cmd_mapping_table[i].cmd, func, strlen(func))) {
			ret_buf = cmd_mapping_table[i].func(param);
			break;
		}
		i++;
	}
	_D("SERVER ret_buf=%s", ret_buf);
	if (func != NULL)
		free(func);
	if (param != NULL)
		free(param);
	return ret_buf;
}

void *__server_init(void *pv)
{
	char *(*pFunc) (char *, int) = __interface_parser;

	_D("[SERVER] adaptor_api_init");

	if (IPC_StartServer(pFunc) < 0) {
		_E("[SERVER] IPC_StartServer Failed");
		return NULL;
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////
// main thread for device api
//////////////////////////////////////////////////////////////////
void *api_main_thread(void *pv)
{
	pthread_t t_server = pthread_self();
	int status;
	int ret;

	_D("device api main_thread ~~~");
/*
	ret = callback_init_connection();
	if (0 != ret)
		_E("[API_SERVER] callback_init_connection error (ret=%d)", ret);
*/
	ret = pthread_create(&t_server, NULL, &__server_init, (void *)NULL);
	if (ret < 0) {
		_E("[API_SERVER] thread create error (ret=%d)", ret);
		return NULL;
	}
	pthread_join(t_server, (void **)&status);

	_D("device api main_thread exit~~~~~~~");

	return NULL;
}
